# lt25.py

🎸 A cross-platform Python module for interacting with the LT25 amplifier made by a certain equine-named guitar manufacture that rhymes with "bender".

## 👋 Introduction

I made this module because I was disappointed by the lack of certain features in both my amp itself as well as the associated desktop app provided by its manufacturer. Failing to find solutions which used modern languages/technologies I could build upon (.NET doesnt count), I made my own package based on the reverse-engineering work others initiated.

### ⚙️ Specifications

This library requires Python 3.8+ to be installed on your system (due to demands of the protobuf 5 module). Additionally, the library only supports HID USB operations (not MIDI)

## 🚀 Quickstart

```
pip install lt25
```

> [!IMPORTANT]
> On linux, hidapi relies on libusb: `sudo apt install libusb-1.0-0-dev libudev-dev pkg-config`

## 💾 Usage

For full documentation of the LT25 class, refer to the [wiki](/wiki)

**Demonstration loop:**

> [!WARNING]
> On some linux distros, `sudo`-mode is required for interacting with USB devices.


```
import time
import lt25

amp = lt25.LT25()
amp.connect()
amp.send_sync_begin()
amp.send_sync_end()
amp.set_preset(5)
amp.set_qa_slots([1,2])
amp.request_firmware_version()
for i in range(5):
    amp.send_heartbeat()
    print(amp.request_qa_slots())
    time.sleep(1)
amp.disconnect()
```

## 🗺️ Roadmap

Compatibility with LtAmp protocol:

- [ ] Audio Status
- [x] Audition Preset (status, start, exit)
- [ ] Clear/rename/shift/swap/save (to/as) preset
- [ ] Connection Status
- [x] Current Preset (request/load)
- [ ] DSP Unit Parameter
- [x] Firmware Version
- [x] Heartbeat
- [ ] ⭐ LT4 Footswitch Mode 
- [ ] Loopback
- [x] Memory usage
- [x] Processor Utilization
- [x] QA Slots
- [x] Sync (Modal Status)
- [x] USB Gain

Other TODOs:
- [ ] Publish to Pypi
- [ ] Continuous Deployment
- [ ] LT25Async/Base Classes
- [ ] Unit Testing
- [ ] Other Amps

## 🛠️ Contributing

> [!NOTE]
> If you need to contribute in a way which updates protocol classes, you can find the original .proto files in Brent Maxwell's [repository](https://github.com/brentmaxwell/LtAmp/). I unfortunately cannot include these files in this module due to copy-left licensing restrictions.

lt25.py is licensed under the permissive MIT license, which means that you may fork, modify, adapt, and redistribute with few restrictions. However, if you wish to contribute your changes back to the base module, please open a pull request. To report bugs, request features, or discuss the project, open an issue or discussion!

## 🙌 Acknowledgements 

- Brent Maxwell ([@brentmaxwell](https://github.com/brentmaxwell)) and his LtAmp .NET libray: his published reverse engineering docs, schemas, proto files, etc. were instrumental in the creation of this Python module.
  - Additionally, some of the goals of his work greatly inspired those of this project.

